from .ImageToGrayscale import ImageToGrayscale
from .GrayscaleToAscii import GrayscaleToAscii
import pyvips
import numpy as np
import time
from PIL import Image

class ImageToAscii:
    def __init__(self, imagePath):
        # convert image in grayscale
        self.imageToGray = ImageToGrayscale(imagePath)
        # than this will convert it into text
        self.grayToAscii = GrayscaleToAscii()

        # and after that this will convert text into image(png, jpg, ...)
        self.asciiToImage = None
        #
    def change_focus_image(self, imagePath):
        self.imageToGray.__init__(imagePath)


    # grayscale array
    def get_grayscaleArray(self):
        return self.imageToGray.arrayImage

    def resize_grayscaleArray_h(self, width, height):
        self.imageToGray.resize_h(width, height)

    def resize_grayscaleArray(self, basewidth):
        self.imageToGray.resize(basewidth)
    # array of ascii chars
    def get_asciiArray(self):
        """
        :return: numpy array with chars
        """
        return self.grayToAscii.arrayAscii

    def save_asciiText(self, filename):
        self.grayToAscii.save(filename)

    # image of pixel but its ascii
    def get_asciiImage(self):
        return self.asciiToImage

    def save_asciiImage(self, filename, scale=1):
        # resize image to original sizes
        self.asciiToImage = self.asciiToImage.resize(scale)
        #  make bg white and text black

        self.asciiToImage.write_to_file(filename)

    def convert(self, time_dublicate = 1,
                textcolor=[0,0,0], backgroundcolor=[255,255,255],
                **kwargs):
        """
        this convert image to Ascii png
        :param time_dublicate: dublicate char in ascii txt (except '\n')
        :param kwargs: for pyvips.Image.text(kwargs)
        :return: None
        """
        self.imageToGray.convert()
        self.grayToAscii.convert(self.get_grayscaleArray(),
                                 times_dublicate=time_dublicate)

        # it s tuple, we convert it into string
        textAscii = str(self.grayToAscii.get_text())
        self.asciiToImage = pyvips.Image.text(
            textAscii, **kwargs,
        )

        # recolor image
        self.asciiToImage = self.asciiToImage.ifthenelse(
            textcolor, backgroundcolor, blend=True
        )


if __name__ == '__main__':
    start_time = time.time()
    # 300 px
    # 2.457428455352783 vs 2.7407114505767822
    # 500 px
    # 11.001618385314941 vs 10.7692391872406 vs
    # todo convert list into numpy array in gray to ascii
    # 600 px
    # 37.03298568725586 vs 42.14830756187439 vs 7.7971556186676025 vs 8.37460708618164 vs 9.05478835105896
    app = ImageToAscii(r'C:\Users\Lenovo\OneDrive\Pictures\Saved Pictures\Alan-Turing-passport-1200x720.jpg')
    origin_width = app.get_grayscaleArray().shape[1]
    new_width = 400
    app.resize_grayscaleArray(new_width)

    app.convert(2, font='consolas')
    im = Image.fromarray(app.get_grayscaleArray())
    new_width = app.get_asciiImage().width
    app.save_asciiImage(r'alan turing.png',
                        scale=origin_width/new_width*0+1)
    print(time.time()-start_time, " second")


from PIL import Image
import numpy as np
from .ImageToGrayscale import ImageToGrayscale
import time

class GrayscaleToAscii:
    # chars for replace pixels
    CHARS_CODE = None
    CHARS = ' .+=*!$#%@'
    def __init__(self):
        # in class static method doesnt work
        if self.CHARS_CODE == None:
            # ' -=*@#%'
            # ' -~/+=*$#%@'
            self.CHARS_CODE = self.create_chars_code(self.CHARS)
        self.arrayAscii = None

    @staticmethod
    def create_chars_code(chars):
        values = ImageToGrayscale.ROUND_BRIGHTNESS.values()
        if len(chars) != len(values):
            raise IndexError(f"values {values} not equel chars '{chars}' ")
        pairs = zip(values, chars)
        return {key:value for key, value in pairs}

    def convert(self, arrayImage, times_dublicate = 1):
        """
        :return: None
        """
        # list: we need to change "col" value for '\n' char
        shape = list(arrayImage.shape)
        shape[1] += 1
        shape[2] = times_dublicate
        self.arrayAscii = np.chararray(shape)
        shape[1] -= 1

        for row in range(shape[0]):
            for col in range(shape[1]):
                # convert pixel into ascii char and
                for dublicate in range(times_dublicate):
                    self.arrayAscii[row][col][dublicate] = self.CHARS_CODE[arrayImage[row][col][0]]
            # last elem
            self.arrayAscii[row][shape[1]][0] = '\n'
            # fill void in '\n' array
            for dublicate in range(1, times_dublicate):
                self.arrayAscii[row][shape[1]][dublicate] = ' '

    def get_text(self):
        return self.arrayAscii.astype('|S1').tostring().decode('utf-8')
    
    def save(self, Asciipath, times_dublicate = 1):
        file = open(Asciipath, 'w')
        arraySave = self.arrayAscii.astype('|S1').tostring().decode('utf-8')
        file.write(arraySave)
        # if times_dublicate != 1:
        #     for col in arraySave:
        #         for char in col:
        #             # dont dublicate '\n' char
        #             if char == '\n':
        #                 file.write(char)
        #                 break
        #             for i in range(times_dublicate):
        #                 file.write(char)
        # else:
        #     file.write(arraySave)
        file.close()

if __name__ == "__main__":
    start_time = time.time()
    img = GrayscaleToAscii(r"C:\Users\Lenovo\PycharmProjects\ConvertImageToASCII\venv\images\aqua_(konosuba).jpg")

    img.convert()
    img.save("ascii/aqua_(ascii).txt")
    print("--- %s seconds ---" % (time.time() - start_time))



from PIL import Image
import numpy as np
import sys
import time

class ImageToGrayscale:
    # ROUND_BRIGHTNESS = {
    #     223: 255,  # white
    #     191: 200,  # white white white black
    #     159: 150,  # white white black
    #     128: 128,  # gray
    #     96: 100,  # black black white
    #     64: 50,  # black black black white
    #     0: 0,  # black
    # }
    # ROUND_BRIGHTNESS = {
    #     239 : 255,
    #     223 : 225,
    #     207 : 210,
    #     191 : 192,
    #     175 : 175,
    #     159 : 160,
    #     143 : 150,
    #     127 : 130,
    #     111 : 112,
    #     95 : 100,
    #     79 : 80,
    #     63 : 64,
    #     47 : 50,
    #     31 : 32,
    #     15 : 16,
    #     0 : 0
    # }
    ROUND_BRIGHTNESS = {
        239: 255,
        207 : 210,
        175: 175,
        143: 150,
        127: 130,
        111: 112,
        79: 80,
        31: 32,
        15: 16,
        0: 0
    }
    def __init__(self, imagePath):
        image = Image.open(imagePath).convert("LA")
        self.arrayImage = np.array(image)
    
    def changeArrayImage(self, imagePath):
        image = Image.open(imagePath).convert("LA")
        self.arrayImage = np.array(image)

    def convert(self):  # roundValues
        shape = self.arrayImage.shape
        for row in range(shape[0]):
            for col in range(shape[1]):
                # round value of pixel
                self.arrayImage[row][col][0] = self.pixelRound(self.arrayImage[row][col][0]);
        # self.image = Image.fromarray(arrayIm)
        # return arrayIm

    @staticmethod
    def pixelRound(value):
        # todo make round_brightness as CONSTANT in class
        # key is compared( > ) value and value is tochange value
        for key in ImageToGrayscale.ROUND_BRIGHTNESS:
            if value >= key:
                return ImageToGrayscale.ROUND_BRIGHTNESS[key]

    def save(self, imagePath):
        image = Image.fromarray(self.arrayImage)
        image.convert("RGB").save(imagePath)

    def resize(self, basewidth):
        image = Image.fromarray(self.arrayImage)
        wpercent = (basewidth / float(image.size[0]))
        hsize = int((float(image.size[1]) * float(wpercent)))
        image = image.resize((basewidth, hsize), Image.ANTIALIAS)
        self.arrayImage = np.array(image)

    def resize_h(self, width, height):
        image = Image.fromarray(self.arrayImage)
        image = image.resize((width, height), Image.ANTIALIAS)
        self.arrayImage = np.array(image)

if __name__ == '__main__':
    image = ImageToGrayscale(r"images\aqua_(konosuba).jpg")
    image.convert()

    im = Image.fromarray(image.arrayImage)
    im.show()

    image.save("method_2.png")
    image.resize(500)

    im = Image.fromarray(image.arrayImage)
    im.show()




from distutils.core import setup
from setuptools import setup

NAME = "ImageToAscii"
DESCRIPTION = 'fast converter image to ascii and fast rendering ascii to image'
URL = "https://github.com/ittybe/imageToAscii"
EMAIL = ""
AUTHOR = "ittybe"
REQUIRES_PYTHON = ">=3.7.0"
VERSION = "1.0.1a"

PY_MODULES = []

REQUIRED = [
    'cffi',
    'numpy',
    'Pillow',
    'pycparser',
    'pyvips',
]
PACKEGES = [
    'ImageToAscii'
]
setup(name=NAME,
      version=VERSION,
      description=DESCRIPTION,
      author=AUTHOR,
      author_email=EMAIL,
      url=URL,
      install_requires=REQUIRED,
      packages=PACKEGES
     )